/*
Copyright 2017-2022 CAcert Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package jobs

import (
	"log"
	"os"
)

type JobScheduler struct {
	infoLogger        *log.Logger
	errorLogger       *log.Logger
	jobs              map[JobIdentifier]Job
	rescheduleChannel chan JobIdentifier
	quitChannel       chan struct{}
}

type SchedulerOption func(scheduler *JobScheduler)

func NewJobScheduler(opts ...SchedulerOption) *JobScheduler {
	rescheduleChannel := make(chan JobIdentifier, 1)

	jobScheduler := &JobScheduler{
		infoLogger:        log.New(os.Stdout, "", 0),
		errorLogger:       log.New(os.Stderr, "", 0),
		jobs:              make(map[JobIdentifier]Job, 2),
		rescheduleChannel: rescheduleChannel,
		quitChannel:       make(chan struct{}),
	}

	for _, o := range opts {
		o(jobScheduler)
	}

	return jobScheduler
}

func SchedulerLog(infoLog, errorLog *log.Logger) SchedulerOption {
	return func(s *JobScheduler) {
		s.infoLogger = infoLog
		s.errorLogger = errorLog
	}
}

func (js *JobScheduler) Schedule() {
	for _, job := range js.jobs {
		job.Schedule()
	}

	for {
		select {
		case jobID := <-js.rescheduleChannel:
			js.jobs[jobID].Schedule()
		case <-js.quitChannel:
			for _, job := range js.jobs {
				job.Stop()
			}

			js.infoLogger.Print("stop job scheduler")

			return
		}
	}
}

func (js *JobScheduler) AddJob(job Job) {
	js.jobs[job.Identifier()] = job
}

func (js *JobScheduler) Quit() {
	js.quitChannel <- struct{}{}
}

func (js *JobScheduler) Reschedule(jobIDs ...JobIdentifier) {
	for i := range jobIDs {
		js.rescheduleChannel <- jobIDs[i]
	}
}
